import { ChangeDetectionStrategy, Component, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { combineLatest, of as ObservableOf } from 'rxjs';
import { catchError, filter, map, switchMap, tap } from 'rxjs/operators';
var TypeaheadSelectorComponent = /** @class */ (function () {
    function TypeaheadSelectorComponent() {
        var _this = this;
        this.headerText = 'Default Header Text';
        this.noValueLabel = 'None';
        this.allowUpdate = true;
        this.allowMultiSelect = false;
        this.closeOnSelect = true;
        this.itemTruncate = 500;
        this.toggleTruncate = 500;
        this.onSelectItem = new EventEmitter();
        this.onOpenSelector = new EventEmitter();
        this.onCloseSelector = new EventEmitter();
        this.searchTermObs = new BehaviorSubject('');
        this.selectedItemsBs = new BehaviorSubject([]);
        this.selectedItemsObs = this.selectedItemsBs.asObservable()
            .pipe(filter(function (i) { return Array.isArray(i); }));
        // this is only used for storing the data temporarily
        // not for visualization
        // we use the data coming in from the input for vusualization
        this._selectedItems = [];
        // This is used to render the UI
        this.selectedItems = [];
        this.searchValue = '';
        this.searching = false;
        this.isOpen = false;
        this.menuItems = this.searchTermObs.asObservable()
            .pipe(tap(function (term) { return _this.searchValue = term; }), switchMap(function (term) {
            if (typeof (_this.dataSource) === 'function'
                && term !== '') {
                return combineLatest(_this.dataSource(term), _this.selectedItemsObs)
                    .pipe(map(function (_a) {
                    var items = _a[0], selectedItems = _a[1];
                    return _this.updateSelection(items, selectedItems);
                }), tap(function (v) { return _this.searching = false; }), catchError(function (err) {
                    _this.searching = false;
                    return ObservableOf([]);
                }));
            }
            else {
                return ObservableOf([]);
            }
        }));
    }
    Object.defineProperty(TypeaheadSelectorComponent.prototype, "selectedItemsSetter", {
        set: function (items) {
            // If selected items are not an array
            // then return an empty array
            // i.e. nothing is selected
            if (!Array.isArray(items)) {
                items = [];
            }
            // If multip selection for this component is off
            // then take only one i.e. first element from items
            if (!this.allowMultiSelect && items.length) {
                this.selectedItems = [items[0]];
            }
            else {
                this.selectedItems = items;
            }
            this._selectedItems = this.selectedItems.slice();
            this.selectedItemsBs.next(this._selectedItems);
        },
        enumerable: true,
        configurable: true
    });
    TypeaheadSelectorComponent.prototype.ngOnInit = function () {
    };
    TypeaheadSelectorComponent.prototype.updateSelection = function (items, selectedItems) {
        var selectedItemsIds = selectedItems.map(function (i) { return i.key; });
        items.forEach(function (item) {
            item.selected = selectedItemsIds.indexOf(item.key) > -1;
        });
        return items;
    };
    TypeaheadSelectorComponent.prototype.onSelect = function (event) {
        if (event.key === null) {
            return;
        }
        if (!this.allowMultiSelect) {
            // empty the array and put the item clicked
            this._selectedItems = [event];
        }
        else {
            // find the index of the clicked item in the selected list
            var index = this._selectedItems.findIndex(function (i) { return i.key === event.key; });
            // if the item clickd is already there then remove it
            if (index > -1) {
                this._selectedItems.splice(index, 1);
            }
            else { // enter the item in the array
                this._selectedItems.push(event);
            }
        }
        this.selectedItemsBs.next(this._selectedItems);
        this.onSelectItem.emit(this._selectedItems);
        if (this.closeOnSelect && !this.allowMultiSelect) {
            this.dropdownRef.closeDropdown();
        }
    };
    TypeaheadSelectorComponent.prototype.onSearch = function (event) {
        // Search on atleast 3 char or numeric
        if (event.trim().length >= 3 || !isNaN(parseInt(event.trim()))) {
            this.searchTermObs.next(event.trim());
            this.searching = true;
        }
        else {
            this.searchTermObs.next('');
        }
    };
    TypeaheadSelectorComponent.prototype.onOpen = function (event) {
        this.isOpen = true;
        this.onOpenSelector.emit();
    };
    TypeaheadSelectorComponent.prototype.onClose = function (event) {
        this.isOpen = false;
        this.dropdownRef.setSearchText('');
        // Empty the search results on close
        this.searchTermObs.next('');
        this.onCloseSelector.emit(this._selectedItems);
    };
    TypeaheadSelectorComponent.decorators = [
        { type: Component, args: [{
                    selector: 'f8-typeahead-selector',
                    template: require('./typeahead-selector.component.html'),
                    styles: [require('./typeahead-selector.component.css').toString()],
                    changeDetection: ChangeDetectionStrategy.OnPush
                },] },
    ];
    /** @nocollapse */
    TypeaheadSelectorComponent.ctorParameters = function () { return []; };
    TypeaheadSelectorComponent.propDecorators = {
        'dropdownRef': [{ type: ViewChild, args: ['dropdown',] },],
        'dataSource': [{ type: Input, args: ['dataSource',] },],
        'headerText': [{ type: Input, args: ['headerText',] },],
        'noValueLabel': [{ type: Input, args: ['noValueLabel',] },],
        'allowUpdate': [{ type: Input, args: ['allowUpdate',] },],
        'allowMultiSelect': [{ type: Input, args: ['allowMultiSelect',] },],
        'closeOnSelect': [{ type: Input, args: ['closeOnSelect',] },],
        'selectedItemsSetter': [{ type: Input, args: ['selectedItems',] },],
        'itemTruncate': [{ type: Input, args: ['itemTruncate',] },],
        'toggleTruncate': [{ type: Input, args: ['toggleTruncate',] },],
        'onSelectItem': [{ type: Output },],
        'onOpenSelector': [{ type: Output },],
        'onCloseSelector': [{ type: Output },],
    };
    return TypeaheadSelectorComponent;
}());
export { TypeaheadSelectorComponent };
//# sourceMappingURL=typeahead-selector.component.js.map